#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>

#include "zzlib.h"

void usage()
{
	printf("usage: dlltest [-d] [-m] srcfilename destfilename");
	exit(1);
}

int main(int argc,char *argv[])
{
	int decompress = 0;
	unsigned int level = 0;
	char *fname_r = NULL;
	char *fname_w = NULL;
	unsigned char *buffer;
	int len;
	struct stat buf_stat;
	FILE *file_r = NULL;
	FILE *file_w = NULL;
	
	while ( ++argv, --argc )
	{
		if ( **argv =='-' || **argv=='/' )
		{
			char *p;
			
			for ( p = *argv + 1 ; *p ; p++ )
			{
				if ( *p == 'd' ) decompress = 1;
				else if ( *p == 'm' ) level = 1;
				else usage();
			}
		}
		else break;
	}
	
	if ( argc >= 1 )
	{
		fname_r = *argv;
		argc--;
		argv++;
	}
	else usage();
	
	if ( argc >= 1 )
	{
		fname_w = *argv;
		argc--;
		argv++;
	}
	else usage();
	
	if ( stat( fname_r, &buf_stat ) == -1 ) 
	{ 
		perror("error:"); 
		exit(1); 
	}
	len = buf_stat.st_size;
	
	if ( (buffer = (unsigned char *)malloc( len + 16 )) == NULL ) 
	{ 
		printf("error: not enough memory\n"); 
		exit(1); 
	}
	
	if ( (file_r = fopen( fname_r, "rb" )) == NULL )
	{
		perror("error:");
		exit(1);
	}
	len = fread( buffer, sizeof(unsigned char), len, file_r );
	fclose( file_r );
	
	
	if ( decompress ) 
	{
		unsigned int len_uncompressed;
		printf("uncompressing... [%s] to [%s]... ", fname_r, fname_w);
        /*
		The 32bit unsigned integer coded in the first four bytes of a compressed block 
		is always greater (<120%) or equal to the size of the uncompressed block, you can 
		use this information if you don't know the size of the uncompressed block.
		*/
		len_uncompressed = *((unsigned int*)buffer);
		buffer = (unsigned char*)realloc( buffer, len_uncompressed );
		len = ZzUncompressBlock( buffer );
	}
	else 
	{
		printf("compressing... [%s] to [%s]... ", fname_r, fname_w);
		len = ZzCompressBlock( buffer, len, level, 0 );
	}

	switch ( len )
	{
	case -1 : printf("not enough memory"); exit(1);
	case -2 : printf("CRC error"); exit(1);
	default : printf("OK"); break;
	}

	
	if ( (file_w = fopen( fname_w, "wb" )) == NULL )
	{
		perror("error:");
		exit(1);
	}
	fwrite( buffer, sizeof(unsigned char), len, file_w );
	fclose( file_w );
	
	return 0;
}
